
# q-expansion of newform 7800.2.a.bf, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7800
weight = 2
poly_data = [-8, -14, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-9, -2, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [-1, 0, 0], [0, 0, 0], [-2, 1, 0], [1, 0, 1], [-1, 0, 0], [0, -1, 0], [1, 1, -1], [-2, 1, 0], [-1, -1, -1], [4, -2, 0], [-1, -2, 1], [1, 2, -1], [-4, 0, 0], [0, -2, 0], [5, -2, -1], [0, 0, 0], [7, 0, 1], [-2, -2, -2], [-1, 2, -1], [-9, -1, -1], [-7, 0, 1], [-2, 2, 2], [5, -4, -1], [-8, -1, 0], [3, 1, -1], [0, 0, 0], [3, 0, -1], [1, 3, 1], [-3, 1, 1], [2, 0, 2], [-9, -1, 1], [0, 2, 2], [-15, -2, 1], [-9, 2, 1], [-2, -2, 2], [8, -2, 2], [3, 4, -1], [4, 0, 0], [4, 2, -2], [-7, 1, -1], [9, 2, -1], [-8, -4, 0], [-4, 1, 0], [2, 2, 0], [-16, 4, 0], [-12, 0, 0], [1, -5, -1], [-4, 6, 0], [-1, -1, -1], [-6, -5, 2], [3, -2, 3], [2, 4, 0], [-9, 1, -3], [1, -3, -3], [1, 3, -1], [-5, 1, -1], [0, 2, -4], [-4, -2, -2], [6, 4, 0], [-4, -4, 0], [18, -2, 0], [-7, 6, 1], [0, 0, 0], [4, 2, 2], [0, 0, 2], [-5, 5, 1], [-14, 6, 0], [5, 2, -3], [-15, -1, 1], [0, 6, 2], [2, -2, 2], [4, 0, -4], [-2, -2, 0], [3, -5, -3], [-14, 0, -2], [-21, 1, -1], [7, -2, 1], [4, 6, 2], [-4, 2, -2], [3, -1, -3], [-1, 1, 3], [-14, 6, 2], [2, -2, 0], [-17, 4, -1], [-11, -2, -3], [-17, -4, 1], [-20, -1, -4], [-17, -4, -3], [-12, 3, 2], [11, -8, -1], [17, 2, 1], [-16, -3, 2], [13, -1, -1], [-17, -1, 1], [5, -5, 3], [-15, 4, -1], [-4, -6, -2], [4, -4, 0], [5, 1, 1], [4, 0, 0], [6, 8, 0], [5, 2, 5], [18, 2, 4], [-21, -2, 3], [-26, -3, 2], [-28, 2, 0], [-24, 4, -2], [4, -4, -4], [-21, 2, -3], [-4, 0, 4], [-7, -6, 3], [-4, 2, -2], [-17, 3, 1], [-16, 4, 0], [0, -4, 6], [-15, 2, 1], [-10, 7, -4], [-22, 0, 0], [9, 3, 3], [-9, 1, 3], [-6, 4, 4], [23, 4, -3], [10, 6, -2], [17, -11, -1], [7, 5, -5], [9, -5, 1], [-14, 2, -2], [0, -6, -4], [23, 2, 1], [19, -1, 5], [-18, 4, 2], [11, 4, 3], [16, 4, -2], [0, 6, -2], [-20, -4, 2], [22, -2, 4], [28, 0, 0], [27, -2, -3], [18, 0, 0], [7, 1, -3], [17, -2, 3], [16, -8, 0], [-24, -6, 4], [30, 0, 0], [-29, 6, 3], [1, 10, -5], [0, -3, -4], [-33, 4, 3], [-2, -12, 2], [-10, 12, 4], [28, -2, -2], [20, -4, 0], [8, 9, 2], [-6, 4, -2], [34, 2, 6], [-15, -8, 1], [-7, -8, -5], [-26, 0, 0], [19, -10, -3], [2, -2, 6], [0, -9, 0], [-5, -7, -3], [-21, 5, 1], [-4, -4, 2], [-20, 8, 0], [11, -8, -5], [-26, 10, 0], [-18, 4, -4], [21, -6, -1], [-2, 6, -6], [-25, -5, -1], [-2, -6, 2], [26, 1, -2], [-1, -8, -1], [10, 2, 4], [5, 9, 3], [7, 2, 1], [-18, -2, -6], [0, 6, -2], [-18, 5, 0], [1, -5, -5], [18, 4, 0], [-38, 0, 0], [18, 4, -2], [-7, 3, 1], [8, 8, 6], [11, 10, 3], [34, 2, 4], [-47, -2, 1], [4, 0, 6], [0, -2, 4], [5, -8, 1], [9, -2, 3], [30, 6, -2], [-10, 5, 2], [-16, 2, 2], [24, -2, 2], [-4, 8, -6], [4, 3, 2], [10, -4, 0], [13, -8, -3], [-25, -8, 5], [-23, 2, 3], [55, 0, -1], [17, -4, -1], [8, 2, -4], [6, 6, -2], [-33, -6, 5], [-5, 0, -9], [-22, -6, 0], [7, -5, -1], [6, 6, -6], [-4, 14, 0], [-10, 14, -2], [-40, 0, -2], [6, -18, 2], [20, 6, -2], [30, 4, 2], [-10, 0, 0], [-29, 6, 1], [-46, 0, 2], [28, -6, 2], [-25, 1, 1], [1, -2, 1], [13, 10, 3], [10, 3, 10], [16, -14, -4], [4, 6, 8], [53, 0, -3], [-6, 14, 4], [19, 4, 7], [10, 6, -2], [-6, 6, -4], [6, -12, 2], [-2, -12, -6], [-22, -6, 4], [-38, -4, 4], [-15, -11, 7], [-26, 18, 2], [18, -2, -2], [7, -8, 3], [50, -6, -2], [27, 5, 7], [20, 4, -6], [16, -4, 8], [-31, -5, -9], [-9, -10, 3], [35, 5, 1], [21, -1, 3], [32, -2, -6], [-42, 8, 0], [0, 2, 8], [27, -12, 1], [-5, -6, -3], [-7, -2, -3], [26, -4, -4], [-45, 0, -1], [20, 10, -2], [-16, -9, -8], [-16, -6, 4], [25, 8, 5], [13, -5, 5], [20, 0, 2], [6, -4, -4], [-25, -6, -1], [13, 4, 11], [-35, 4, -1], [23, 10, -1], [6, 12, -4], [12, -14, 2], [1, -8, -3], [-22, 9, -2], [-20, 12, -4], [-18, 0, 0], [8, 9, -6], [-2, -6, 2], [-13, 5, -1], [38, 6, 8], [-35, -3, 3], [20, 3, -6], [36, 0, 4], [-3, 15, -5], [-9, -5, -9], [2, -2, -6], [-22, 2, -2], [2, 14, 0], [-6, 6, -8], [10, -12, -6], [0, 4, -2], [13, 11, -3], [-24, -12, 4], [28, 4, -6], [-26, 6, 2], [6, -4, -4], [21, 5, 9], [-28, 20, 4], [-35, -4, 3], [41, -1, 3], [3, -16, -1], [4, -14, 2], [-30, -6, 8], [-11, -4, -3], [-1, -6, -1], [-8, -8, 4], [-16, 13, 4], [37, -12, 1], [23, 14, -3], [10, 14, 2], [-36, -8, 2], [-10, 1, 0], [-42, -8, -4], [4, -10, 6], [16, -8, 4], [34, 10, 2], [-49, 4, 5], [19, 12, -5], [-9, 14, -1], [-29, 7, 3], [28, 4, 2], [-30, -6, -6], [-33, 7, -1], [33, -11, 5], [-40, -5, -2], [-38, -8, 0], [-33, 2, -7], [11, -5, -3], [1, -10, 1], [-4, 15, -6], [-64, -2, -6], [-19, 9, -7], [-7, 4, -1], [48, 8, 0], [22, -2, 2], [63, -11, -3], [-36, 6, 0], [5, 13, 1], [-10, -2, 8], [-16, -2, -6], [-42, 4, 6], [3, 16, -7], [-20, 1, 0], [3, -18, 5], [-4, -8, -4], [-20, 6, -6], [41, 3, 3], [-13, 16, 5], [-31, -2, -7], [10, -10, 2], [13, 12, 3], [-5, 19, 5], [-24, 21, 4], [41, -10, 3], [-20, -2, -8], [55, 10, -3], [39, 3, 7], [-17, -14, 7], [24, -20, 0], [-37, 3, 3], [16, -6, 8], [18, -4, -8], [-68, 0, 2], [49, -9, -1], [22, -2, 2], [46, -4, 2], [-26, 10, 4], [-2, -2, -4], [-20, -9, 2], [31, 4, 5], [-33, -1, -7], [-18, -2, 6], [-8, 0, -4], [-5, 15, -5], [15, -12, -9], [24, -20, -2], [-47, 2, -3], [46, 0, -2], [-10, 21, 2], [-46, 6, -4], [-28, -6, 2], [-40, 10, 2], [8, 2, -6], [1, 3, -1], [6, 8, 4], [-3, 7, 7], [62, 6, -2], [-30, 8, -6], [-16, 8, 6], [-70, -2, -6], [-40, -9, -2], [-12, -4, 2], [-6, 16, -4], [37, 0, 5], [24, 0, 12], [-13, -10, -5], [32, 6, 6], [28, 6, 16], [-30, -4, 0], [-9, -12, -5], [2, 0, -8], [-23, -7, -3], [16, -11, -4], [22, 7, -4], [-16, 4, -2], [-17, -12, 5], [15, -4, 7], [-28, 6, -2], [6, -8, -8], [-19, 8, 9], [13, -10, 1], [0, -2, -2], [-25, 6, 1], [1, -6, 9], [41, 18, -7], [42, 4, 12], [15, -1, 7], [-11, 6, -11], [-18, -12, -2], [16, 4, -10], [46, -9, -4], [23, -17, -9], [17, 0, -9], [-6, -18, 10], [11, -23, 1], [-25, 15, 7], [24, 16, 2], [-37, 4, -9], [-29, -12, -3], [23, 11, 1], [-6, 0, 2], [-8, -18, 2], [59, -14, -1], [27, -6, -5], [2, 8, -2], [29, 6, -9], [2, -8, 8], [12, -18, 6], [-21, -9, -13], [52, 8, -8], [58, 6, -2], [34, 14, -2], [-21, 10, 9], [27, -15, -9], [26, 4, -6], [-36, 10, -2], [-23, -11, -3], [-14, 16, -10], [-12, 2, 12], [-40, 2, -6], [-22, -12, 4], [-61, -8, -1], [46, -6, 6], [0, 10, -12], [-54, 4, 4], [30, -9, 6], [-3, 7, -3], [-9, -5, 7], [33, -7, -1], [-24, -6, 2], [22, -16, -4], [-85, 2, 3], [-44, 8, 4], [-9, -23, 5], [29, 10, 3], [-49, 24, 3], [-30, 7, -6], [-65, -4, -9], [-16, -2, -4], [-38, 8, 8], [12, 4, 0], [7, 13, 9], [56, -14, -4], [-25, -10, -1], [-10, 18, 0]]
